require 'rexml/document'

module WorkTimer

class ApplicationProperties
public
   attr_reader :propertyHash
   
   def ApplicationProperties::instance
      if @@theInstance == nil then
         @@theInstance = ApplicationProperties.new
      end
      
      return @@theInstance
   end
   
   def close
      begin                  
         propDoc = REXML::Document.new
                  
         propDoc.add_element("WorkTimeRecorderProperties")
         
         propertyHash.each_pair {
            |propName, propValues|

            anElement = REXML::Element.new("Property")
            
            anElement.add_attribute("name", propName)
            
            propValues.each_pair {
               |attrName, attrValue|
               anElement.add_attribute(attrName, attrValue)
            }
            
            propDoc.root.add_element(anElement)
         }
         
         theFile = File.new("AppConfig.xml", "w")
         
         #write the document
         propDoc.write(theFile,0)
         
         theFile.close

      rescue
         #do nothing
      end
   
   end
   
private
   @@theInstance = nil

   def initialize
      # open the properties file and load the application properties
      begin
         @propertyHash = Hash.new
         
         theFile = File.new("AppConfig.xml", "r")
         
         propDoc = REXML::Document.new(theFile)
         
         theFile.close
         
         propDoc.root.elements.each {
            |property|
            
            # basically, just want to create a new
            # key based on the property name and then
            # iterate over the attributes for the property.
            # Each attribute will be stored in a sub hash
            # associated with each property name
            @propertyHash[property.attribute("name").value] = Hash.new
            
            property.attributes.keys.find_all {|key| key != "name"}.each {
               |name|
               
               subHash = @propertyHash[property.attribute("name").value]
               
               subHash[name] = property.attribute(name).value
            }
         }
      rescue
         populateDefaultProperties
      end
   end
   
   def populateDefaultProperties
      @propertyHash["DefaultExportDirectory"] = Hash.new
      @propertyHash["DefaultExportType"] = Hash.new
      @propertyHash["ReportDialogInfo"] = Hash.new
      @propertyHash["TimeEditorDialogInfo"] = Hash.new
      @propertyHash["ActivityDialogInfo"] = Hash.new
      @propertyHash["WorkTimeRecorderInfo"] = Hash.new
      
      @propertyHash["DefaultExportDirectory"]["value"] = "."
      @propertyHash["DefaultExportType"]["value"] = "0"

      @propertyHash["ActivityDialogInfo"]["width"] = "268"
      @propertyHash["ActivityDialogInfo"]["height"] = "91"
      
      @propertyHash["ReportDialogInfo"]["width"] = "290"
      @propertyHash["ReportDialogInfo"]["height"] = "400"
      
      @propertyHash["TimeEditorDialogInfo"]["width"] = "305"
      @propertyHash["TimeEditorDialogInfo"]["height"] = "400"

      @propertyHash["WorkTimeRecorderInfo"]["width"] = "250"
      @propertyHash["WorkTimeRecorderInfo"]["height"] = "350"
      @propertyHash["WorkTimeRecorderInfo"]["x"] = "100"
      @propertyHash["WorkTimeRecorderInfo"]["y"] = "100"
   end
end

if __FILE__ == $0
   ApplicationProperties.instance
   
   ApplicationProperties.instance.propertyHash.each_pair {
      |name,value|
      puts "Property Name: #{name}:"
      
      value.each_pair {
         |aName, aValue|
         puts "--->Attribute: #{aName}, Value: #{aValue}"
      }
   }
end

end