require 'TimerEvent'
require 'EventContainer'
require 'rexml/document'

module WorkTimer

class TimeClock
   attr_reader :activityList
   attr_reader :eventFileName
   attr_reader :activityFileName
   attr_reader :openActivities
   attr_accessor :defaultActivity
   attr_reader :eventList
   
   def initialize(activityFile, eventFile)
      @activityFileName = activityFile
      @eventFileName = eventFile
      @defaultActivity = nil
                        
      @openActivities = Array.new
      
      @activityList = Array.new

      @eventList = Array.new
      
      readActivityFile
            
      readEventFile
   end
   
   def writeActivityFile
      # create an xml document with the activities
      actDoc = REXML::Document.new
      actDoc.add_element("Activities")
      
      #now loop through the activity list and
      #add them to the document
      @activityList.each {
         |activity|
         
         isDefault = false
                  
         if @defaultActivity == activity then
            isDefault = true
         end
         
         anElement = REXML::Element.new("Activity")
         
         anElement.add_attribute("name", activity)
         
         anElement.add_attribute("is_default", isDefault.to_s)
 
         actDoc.root.add_element(anElement)
      }
      
      theFile = File.new(@activityFileName, "w")
      
      #write the document
      actDoc.write(theFile,0)
      
      theFile.close
   end
   
   def readActivityFile
      begin
         theFile = File.new(@activityFileName)
         
         actDoc = REXML::Document.new(theFile)
         
         theFile.close
         
         actDoc.root.elements.each {
            |activity|
            actName = activity.attribute("name").to_s
                     
            if activity.attribute("is_default").to_s.downcase == "true" then
               @defaultActivity = actName
            end
            
            @activityList.push(actName)
         }
      rescue
         #do nothing
      end
   end
   
   def closeAllOpenActivities
      tempList = @openActivities.dup
      
      tempList.each {
         |item|
         stopActivity(item)
      }   
   end
   
   def stopTimeClock
      closeAllOpenActivities            
   end
   
   def addActivity(activityName)
      @activityList.push(activityName)
      writeActivityFile
   end
   
   def deleteActivity(activityName)
      @activityList.delete(activityName)
      @activityList.each {
         |item|
      }
      writeActivityFile
   end
   
   def renameActivity(oldName, newName)
      @activityList.each_index {
         |i|
         if @activityList[i] == oldName then
            @activityList[i] = newName
            break
         end
      }
      writeActivityFile
   end
      
   def writeEventFile
      aDoc = REXML::Document.new
      aDoc.add_element("Events")
      
      @eventList.each {
         |event|
         aDoc.root.add_element(
            createEventElement(event))
      }
      
      aFile = File.new(@eventFileName, "w")
      aDoc.write(aFile,0)
      aFile.close
   end
   
   def createEventElement(anEvent)
      anElement = REXML::Element.new("Event")
      anElement.add_attribute("type", anEvent.eventType)
      anElement.add_attribute("time", anEvent.eventTime)
      anElement.add_attribute("description", anEvent.eventDescription)   
      return anElement
   end
   
   def readEventFile
      #reads the events in the event file into the
      #eventList
      aDoc = nil
      
      @eventList.clear
      
      begin
         aFile = File.new(@eventFileName)
         aDoc = REXML::Document.new(aFile)
         aFile.close
      rescue
         aDoc = REXML::Document.new
         aDoc.add_element("Events")         
      end
      
      aDoc.root.elements.each {
         |event|      
         anEvent = TimerEvent.new(
                      event.attribute("type").to_s,
                      event.attribute("description").to_s,
                      event.attribute("time").to_s.to_i)
                      
         @eventList.push(anEvent)
      }
   end
   
   def setDefaultActivity(activityName)
      if @activityList.member?(activityName) then
         @defaultActivity = activityName
      end
      writeActivityFile
   end
   
   def startActivity(activityName)
      #close all open activities
      closeAllOpenActivities
      
      @eventList.push(
         TimerEvent.new("Start", activityName, Time.now.to_i))
      
      @openActivities.push(activityName)
      
      @openActivities.uniq!
      
      writeEventFile
   end
   
   def stopActivity(activityName)      
      #add a stop event      
      @eventList.push(
         TimerEvent.new("Stop", activityName, Time.now.to_i))
      
      @openActivities.delete(activityName)
      
      writeEventFile
   end

   def createEventHash(startTime, endTime)
      eventHash = Hash.new
      
      @eventList.each {
         |anEvent|      
         
         if anEvent.eventTime < startTime or
            anEvent.eventTime > endTime then
            next
         end
         
         if not eventHash.has_key?(anEvent.eventDescription) then
            eventHash[anEvent.eventDescription] = EventContainer.new
         end
         
         eventHash[anEvent.eventDescription].addEvent(anEvent)
      }            
      return eventHash   
   end
      
end

end