module WorkTimer

require 'rexml/document'

class XmlExportWriter

public

   def export(filename, theTimeClock, startTime, endTime)
      # create a new document
      actDoc = REXML::Document.new
      actDoc.add_element("Report")

      eventHash = theTimeClock.createEventHash(startTime, endTime)
      
      #Divide the time interval into days
      numDays = (endTime - startTime) / (3600 * 24)
            
      if (endTime - startTime) % (3600 * 24) > 0
         numDays += 1
      end
      
      #create an array to hold the totals
      dailyTotals = Array.new(numDays,0)

      grandTotal = 0
      
      startTimeString = Time.at(startTime).strftime("%m/%d/%Y")
      
      # note: we're actually producing a report for the interval
      # [startTime, endTime) so we should subtract a second from
      # the end time string so that we don't lead the user to
      # think that the report includes the next day
      endTimeString = Time.at(endTime-1).strftime("%m/%d/%Y")
      
      actDoc.root.add_attribute("type", "DailySummary")
      actDoc.root.add_attribute("startDate", startTimeString)
      actDoc.root.add_attribute("endDate", endTimeString)
      
      reportTitleElement = REXML::Element.new("ReportTitle", actDoc.root)
      reportTitleElement.text = 
         "Activity Report (#{startTimeString} - #{endTimeString})"
      
      #get a summary of activities for each day
      1.upto(numDays) {
         |dayCount|
         dayStart = startTime + (dayCount - 1) * 3600 * 24
         dayEnd = dayStart + 3600 * 24
                  
         activityTotalsElement = REXML::Element.new("ActivityTotals")
         activityTotalsElement.add_attribute(
            "date",
            Time.at(dayStart).strftime("%m/%d/%Y"))
                                                 
         eventHash.each_pair {
            |description, eventContainer|
            
            activityElement = REXML::Element.new("Activity")
            activityElement.add_attribute("name", description)
            
            #get the amount of time spent in hours
            totalEventTime = eventContainer.computeTimeSpent(dayStart, dayEnd)
            
            totalHrs      = totalEventTime / 3600.0
            
            dailyTotals[dayCount - 1] += totalHrs

            grandTotal += totalHrs
            
            if totalEventTime > 0 then
               activityElement.add_attribute("total", sprintf("%.2f", totalHrs))
               activityTotalsElement.add_element(activityElement)
            end            
         }  
         
         if dailyTotals[dayCount - 1] > 0 then
            activityTotalsElement.add_attribute(
               "total",
               sprintf("%.2f", dailyTotals[dayCount - 1].to_s))
               
            actDoc.root.add_element(activityTotalsElement)
         end
      }
      
      # add the grandTotal attribute
      actDoc.root.add_attribute("total", sprintf("%.2f", grandTotal))
      
      theFile = File.open(filename, "w")
      
      #write the document
      actDoc.write(theFile,0)
      
      theFile.close

   end
end


end